import json
import sys
from pathlib import Path
from typing import Callable, Optional, Union, Tuple
from decimal import Decimal


import numpy as np

import talib
from datetime import datetime, time, timedelta
from .模块_常数 import 类_周期
from .模块_对象 import 类_K线数据,类_行情数据

# from .模块_常数 import 类_交易所,类_周期
from xmpy.包_交易核心.模块_常数 import 类_交易所

if sys.version_info >= (3, 9):
    from zoneinfo import ZoneInfo, available_timezones              # noqa
else:
    from backports.zoneinfo import ZoneInfo, available_timezones    # noqa

def _获取交易目录(文件夹名称: str) -> Tuple[Path, Path]:
    """获取交易平台运行时目录"""
    # 获取当前工作目录
    当前路径: Path = Path.cwd()

    # 拼接临时目录路径
    临时路径: Path = 当前路径.joinpath(文件夹名称)

    # 检查是否存在.vntrader目录
    if 临时路径.exists():
        return 当前路径, 临时路径

    # 获取用户主目录
    用户目录: Path = Path.home()
    临时路径 = 用户目录.joinpath(文件夹名称)

    # 创建不存在的目录
    if not 临时路径.exists():
        临时路径.mkdir()

    return 用户目录, 临时路径


# 初始化目录配置
交易目录, 临时目录 = _获取交易目录(".xmpy文件保存")
sys.path.append(str(交易目录))  # 添加至Python路径


def 获取文件路径(文件名称: str) -> Path:
    """获取临时目录下的文件完整路径"""
    return 临时目录.joinpath(文件名称)


def 加载json文件(文件名称: str) -> dict:
    """从临时目录加载JSON文件数据"""
    文件路径: Path = 获取文件路径(文件名称)

    if 文件路径.exists():
        with open(文件路径, mode="r", encoding="UTF-8") as 文件对象:
            数据字典: dict = json.load(文件对象)
        return 数据字典
    else:
        # 文件不存在时创建空文件
        保存json文件(文件名称, {})
        return {}


def 保存json文件(文件名称: str, 数据字典: dict) -> None:
    """保存数据到临时目录的JSON文件"""
    文件路径: Path = 获取文件路径(文件名称)
    with open(文件路径, mode="w+", encoding="UTF-8") as 文件对象:
        json.dump(
            数据字典,
            文件对象,
            indent=4,           # 4空格缩进
            ensure_ascii=False  # 支持非ASCII字符
        )

def 保存文本文件(文件名称: str, 内容) -> None:
    """将任意内容保存为临时目录中的 .txt 文件"""
    文件路径: Path = 获取文件路径(文件名称 + ".txt")

    # 将内容转换为字符串
    if isinstance(内容, (dict, list, tuple)):
        # 如果是结构化数据，用 JSON 格式美化输出（便于阅读）
        内容字符串 = json.dumps(内容, indent=4, ensure_ascii=False)
    else:
        # 其他类型直接转为字符串
        内容字符串 = str(内容)

    with open(文件路径, mode="a", encoding="UTF-8") as 文件对象:
        文件对象.write(内容字符串 + "\n")

def 获取目录路径(目录名称: str) -> Path:
    """获取临时目录下的指定子目录路径"""
    目录路径: Path = 临时目录.joinpath(目录名称)

    if not 目录路径.exists():
        目录路径.mkdir()

    return 目录路径

def 虚拟方法(func: Callable) -> Callable:
    """
    标记函数为"可重写"的虚拟方法
    所有基类应使用此装饰器或@abstractmethod来标记子类可重写的方法
    """
    return func

def 提取合约代码(合约标识: str) -> Tuple[str, '类_交易所']:
    """
    :return: (代码, 交易所)
    """

    代码, 交易所字符串 = 合约标识.rsplit(".", 1)
    return 代码, 类_交易所[交易所字符串]

def 四舍五入到指定值(数值: float, 目标值: float) -> float:
    """
    根据目标值四舍五入价格。
    将价格按最小变动单位四舍五入

    :param 数值: 需要处理的价格数值
    :param 目标值: 最小价格变动单位（如0.5）
    :return: 四舍五入后的标准价格
    """
    数值: Decimal = Decimal(str(数值))
    目标值: Decimal = Decimal(str(目标值))
    四舍五入结果: float = float(int(round(数值 / 目标值)) * 目标值)
    return 四舍五入结果

def 提取合约前缀(合约代码: str) -> str:
    """从合约代码中提取品种部分（去掉数字）"""
    # 使用正则表达式匹配字母部分
    match = re.match(r'([a-zA-Z]+)', 合约代码)
    if match:
        return match.group(1)

class 类_K线生成器:
    """
    K线合成器功能：
    1. 从Tick数据合成1分钟K线
    2. 从基础K线合成多周期K线（分钟/小时/日线）
    注意：
    1. 分钟周期必须为60的约数
    2. 小时周期可为任意整数
    """

    def __init__(
        self,
        K线回调: Callable,
        窗口周期: int = 0,
        窗口回调: Callable = None,
        周期类型: 类_周期 = 类_周期.一分钟,
        日结束时间: time = None
    ) -> None:
        self.当前K线: 类_K线数据 = None
        self.K线回调 = K线回调

        self.周期类型 = 周期类型
        self.周期计数: int = 0

        self.小时K线缓存: 类_K线数据 = None
        self.日K线缓存: 类_K线数据 = None

        self.窗口大小 = 窗口周期
        self.窗口K线缓存: 类_K线数据 = None
        self.窗口回调 = 窗口回调

        self.最后Tick缓存:类_行情数据  = None
        self.日结束时间 = 日结束时间

        if self.周期类型 == 类_周期.日线 and not self.日结束时间:
            raise ValueError("日线合成必须指定收盘时间")

    def 更新Tick(self, tick: 类_行情数据) -> None:
        """处理Tick更新"""
        新周期标志 = False

        if not tick.最新价:
            return

        if not self.当前K线:
            新周期标志 = True
        elif (
                (self.当前K线.时间戳.minute != tick.时间戳.minute)
                or (self.当前K线.时间戳.hour != tick.时间戳.hour)
        ):
            self.当前K线.时间戳 = self.当前K线.时间戳.replace(second=0, microsecond=0)
            self.K线回调(self.当前K线)
            新周期标志 = True

        if 新周期标志:
            self.当前K线 = 类_K线数据(
                代码=tick.代码,
                交易所=tick.交易所,
                周期=类_周期.一分钟,
                时间戳=tick.时间戳,
                网关名称=tick.网关名称,
                开盘价=tick.最新价,
                最高价=tick.最新价,
                最低价=tick.最新价,
                收盘价=tick.最新价,
                持仓量=tick.持仓量
            )
        else:
            self.当前K线.最高价 = max(self.当前K线.最高价, tick.最新价)
            if tick.最高价 > self.最后Tick缓存.最高价:
                self.当前K线.最高价 = max(self.当前K线.最高价, tick.最高价)

            self.当前K线.最低价 = min(self.当前K线.最低价, tick.最新价)
            if tick.最低价 < self.最后Tick缓存.最低价:
                self.当前K线.最低价 = min(self.当前K线.最低价, tick.最低价)

            self.当前K线.收盘价 = tick.最新价
            self.当前K线.持仓量 = tick.持仓量
            self.当前K线.时间 = tick.时间戳

        if self.最后Tick缓存:
            成交量变动 = max(tick.成交量 - self.最后Tick缓存.成交量, 0)
            self.当前K线.成交量 += 成交量变动

            成交额变动 = max(tick.成交额 - self.最后Tick缓存.成交额, 0)
            self.当前K线.成交额 += 成交额变动

        self.最后Tick缓存 = tick

    def 更新K线(self, bar: 类_K线数据) -> None:
        """处理K线更新"""
        if self.周期类型 == 类_周期.一分钟:
            self._处理分钟窗口(bar)
        elif self.周期类型 == 类_周期.一小时:
            self._处理小时窗口(bar)
        else:
            self._处理日线窗口(bar)

    def _处理分钟窗口(self, bar: 类_K线数据) -> None:
        """分钟级窗口处理"""
        if not self.窗口K线缓存:
            基准时间: datetime = bar.时间戳.replace(second=0, microsecond=0)
            self.窗口K线缓存 = 类_K线数据(
                代码=bar.代码,
                交易所=bar.交易所,
                时间戳=基准时间,
                网关名称=bar.网关名称,
                开盘价=bar.开盘价,
                最高价=bar.最高价,
                最低价=bar.最低价
            )
        else:
            self.窗口K线缓存.最高价 = max(self.窗口K线缓存.最高价, bar.最高价)
            self.窗口K线缓存.最低价 = min(self.窗口K线缓存.最低价, bar.最低价)

        self.窗口K线缓存.收盘价 = bar.收盘价
        self.窗口K线缓存.成交量 += bar.成交量
        self.窗口K线缓存.成交额 += bar.成交额
        self.窗口K线缓存.持仓量 = bar.持仓量

        if not (bar.时间戳.minute + 1) % self.窗口大小:
            self.窗口回调(self.窗口K线缓存)
            self.窗口K线缓存 = None

    def _处理小时窗口(self, bar: 类_K线数据) -> None:
        """小时级窗口处理"""
        if not self.小时K线缓存:
            基准时间: datetime = bar.时间戳.replace(minute=0, second=0, microsecond=0)
            self.小时K线缓存 = 类_K线数据(
                代码=bar.代码,
                交易所=bar.交易所,
                时间戳=基准时间,
                网关名称=bar.网关名称,
                开盘价=bar.开盘价,
                最高价=bar.最高价,
                最低价=bar.最低价,
                收盘价=bar.收盘价,
                成交量=bar.成交量,
                成交额=bar.成交额,
                持仓量=bar.持仓量
            )
            return

        完成K线 = None

        if bar.时间戳.minute == 59:
            self.小时K线缓存.最高价 = max(self.小时K线缓存.最高价, bar.最高价)
            self.小时K线缓存.最低价 = min(self.小时K线缓存.最低价, bar.最低价)
            self.小时K线缓存.收盘价 = bar.收盘价
            self.小时K线缓存.成交量 += bar.成交量
            self.小时K线缓存.成交额 += bar.成交额
            self.小时K线缓存.持仓量 = bar.持仓量

            完成K线 = self.小时K线缓存
            self.小时K线缓存 = None
        elif bar.时间戳.hour != self.小时K线缓存.时间戳.hour:
            完成K线 = self.小时K线缓存
            基准时间: datetime = bar.时间戳.replace(minute=0, second=0, microsecond=0)
            self.小时K线缓存 = 类_K线数据(
                代码=bar.代码,
                交易所=bar.交易所,
                时间戳=基准时间,
                网关名称=bar.网关名称,
                开盘价=bar.开盘价,
                最高价=bar.最高价,
                最低价=bar.最低价,
                收盘价=bar.收盘价,
                成交量=bar.成交量,
                成交额=bar.成交额,
                持仓量=bar.持仓量
            )
        else:
            self.小时K线缓存.最高价 = max(self.小时K线缓存.最高价, bar.最高价)
            self.小时K线缓存.最低价 = min(self.小时K线缓存.最低价, bar.最低价)
            self.小时K线缓存.收盘价 = bar.收盘价
            self.小时K线缓存.成交量 += bar.成交量
            self.小时K线缓存.成交额 += bar.成交额
            self.小时K线缓存.持仓量 = bar.持仓量

        if 完成K线:
            self._处理完成小时K线(完成K线)

    def _处理完成小时K线(self, bar: 类_K线数据) -> None:
        """完成小时K线后续处理"""
        if self.窗口大小 == 1:
            self.窗口回调(bar)
        else:
            if not self.窗口K线缓存:
                self.窗口K线缓存 = 类_K线数据(
                    代码=bar.代码,
                    交易所=bar.交易所,
                    时间戳=bar.时间戳,
                    网关名称=bar.网关名称,
                    开盘价=bar.开盘价,
                    最高价=bar.最高价,
                    最低价=bar.最低价
                )
            else:
                self.窗口K线缓存.最高价 = max(self.窗口K线缓存.最高价, bar.最高价)
                self.窗口K线缓存.最低价 = min(self.窗口K线缓存.最低价, bar.最低价)

            self.窗口K线缓存.收盘价 = bar.收盘价
            self.窗口K线缓存.成交量 += bar.成交量
            self.窗口K线缓存.成交额 += bar.成交额
            self.窗口K线缓存.持仓量 = bar.持仓量

            self.周期计数 += 1
            if not self.周期计数 % self.窗口大小:
                self.周期计数 = 0
                self.窗口回调(self.窗口K线缓存)
                self.窗口K线缓存 = None

    def _处理日线窗口(self, bar: 类_K线数据) -> None:
        """日线级窗口处理"""
        if not self.日K线缓存:
            self.日K线缓存 = 类_K线数据(
                代码=bar.代码,
                交易所=bar.交易所,
                时间戳=bar.时间戳,
                网关名称=bar.网关名称,
                开盘价=bar.开盘价,
                最高价=bar.最高价,
                最低价=bar.最低价
            )
        else:
            self.日K线缓存.最高价 = max(self.日K线缓存.最高价, bar.最高价)
            self.日K线缓存.最低价 = min(self.日K线缓存.最低价, bar.最低价)

        self.日K线缓存.收盘价 = bar.收盘价
        self.日K线缓存.成交量 += bar.成交量
        self.日K线缓存.成交额 += bar.成交额
        self.日K线缓存.持仓量 = bar.持仓量

        if bar.时间戳.time() == self.日结束时间:
            self.日K线缓存.时间 = bar.时间戳.replace(hour=0, minute=0, second=0, microsecond=0)
            self.窗口回调(self.日K线缓存)
            self.日K线缓存 = None

    def 立即生成(self) -> Optional[类_K线数据]:
        """强制生成当前K线"""
        if self.当前K线:
            self.当前K线.时间 = self.当前K线.时间.replace(second=0, microsecond=0)
            self.K线回调(self.当前K线)
            result = self.当前K线
            self.当前K线 = None
            return result
        return None

def 爬取主力合约表格():
    # 固定文件名
    文件名称 = "主力合约记录.json"

    # 获取当天日期
    今天日期 = datetime.now().strftime("%Y-%m-%d")

    try:
        # 尝试加载现有JSON文件
        现有数据 = 加载json文件(文件名称)

        # 检查日期是否为今天
        if 现有数据 and 现有数据.get("日期") == 今天日期:
            print(f"今天({今天日期})的数据已存在，跳过爬取")
            return
    except Exception as e:
        # 文件不存在或其他错误，继续执行爬取
        print(f"加载现有文件失败: {e}，开始爬取新数据")

    # 执行爬取
    网址 = "http://openctp.cn/fees.html"

    try:
        # 发送请求获取网页内容
        响应 = requests.get(网址)
        响应.encoding = 'utf-8'  # 设置编码

        if 响应.status_code == 200:
            解析器 = BeautifulSoup(响应.text, 'html.parser')

            # 找到表格
            表格 = 解析器.find('table', {'id': 'fees_table'})

            if 表格:
                # 获取黄色背景的行数据
                合约代码列表 = []
                表格主体 = 表格.find('tbody')

                for 行 in 表格主体.find_all('tr'):
                    单元格列表 = 行.find_all('td')

                    # 确保有足够的列
                    if len(单元格列表) > 1:
                        交易所 = 单元格列表[0].text.strip()  # 第一列是交易所
                        合约代码单元格 = 单元格列表[1]  # 第二列是合约代码

                        # 只检查合约代码单元格是否有黄色背景
                        单元格样式 = 合约代码单元格.get('style')
                        if 单元格样式 and 'background-color:yellow' in 单元格样式:
                            合约代码 = 合约代码单元格.text.strip()
                            拼接结果 = f"{合约代码}.{交易所}"
                            合约代码列表.append({
                                "合约代码": 合约代码,
                                "交易所": 交易所,
                                "完整代码": 拼接结果
                            })

                # 按交易所分组
                分组数据 = {}
                for 合约 in 合约代码列表:
                    交易所 = 合约["交易所"]
                    if 交易所 not in 分组数据:
                        分组数据[交易所] = []
                    分组数据[交易所].append(合约["完整代码"])

                # 构建输出数据
                输出数据 = {
                    "日期": 今天日期,
                    "数据": 分组数据
                }

                # 打印结果
                if 合约代码列表:
                    # 调用外部保存函数
                    保存json文件(文件名称, 输出数据)
                    print(f"\n今日主力合约数据已保存到: {文件名称}")
                else:
                    print("未找到符合条件的合约数据")

            else:
                print("未找到表格")
        else:
            print(f"请求失败，状态码: {响应.status_code}")

    except Exception as 异常:
        print(f"发生错误: {异常}")

def 处理合约信息(交易所名称: str = "全部") -> None:
    """
    处理合约数据

    参数:
        交易所名称: 交易所名称，默认为"全部"，可选CZCE、GFEX、CFFEX、DCE、SHFE、INE

    返回:
        主力合约列表
    """
    # 加载JSON文件
    文件名称 = "主力合约记录.json"
    现有数据 = 加载json文件(文件名称)
    数据字典 = 现有数据['数据']

    # 验证交易所名称
    if 交易所名称 != "全部" and 交易所名称 not in 数据字典:
        raise ValueError(f"错误: 未找到交易所 {交易所名称}。可选的交易所有: {', '.join(数据字典.keys())}")

    # 选择要处理的交易所
    目标交易所列表 = [交易所名称] if 交易所名称 != "全部" else 数据字典.keys()

    # 收集合约代码
    主力合约列表 = []
    for 交易所 in 目标交易所列表:
        合约列表 = 数据字典[交易所]
        主力合约列表.extend(合约列表)

    return 主力合约列表


if __name__ == "__main__":
    # 使用示例
    合约标识 = "TA506.郑商所"
    代码, 交易所 = 提取合约代码(合约标识)
    print(f"代码: {代码}, 交易所对应字符串: {交易所}")


from collections.abc import Callable
from itertools import product
from concurrent.futures import ProcessPoolExecutor
from random import random, choice
from time import perf_counter
from multiprocessing import get_context
from multiprocessing.context import BaseContext
from multiprocessing.managers import DictProxy
from _collections_abc import dict_keys, dict_values, Iterable

from deap import base, tools, algorithms       # type: ignore
from tqdm import tqdm

输出函数 = Callable[[str], None]

class 类_优化设置:
    """
    用于运行优化的设置
    """

    def __init__(self) -> None:
        """"""
        self.参数: Dict[str, List] = {}
        self.优化目标: str = ""

    def 添加参数(
        self,
        参数名: str,
        起始值: float,
        结束值: float = None,
        步长: float = None
    ) -> tuple[bool, str]:
        """"""
        if 结束值 is None and 步长 is None:
            self.参数[参数名] = [起始值]
            return True, "固定参数添加成功"

        if 起始值 >= 结束值:
            return False, "参数优化起始点必须小于终止点"

        if 步长 <= 0:
            return False, "参数优化步进必须大于0"

        当前值: float = 起始值
        值列表: List[float] = []

        while 当前值 <= 结束值 + 1e-12:
            值列表.append(round(当前值, 6))  # 保留 6 位小数
            当前值 += 步长

        self.参数[参数名] = 值列表

        return True, f"范围参数添加成功，数量{len(值列表)}"

    def 设置优化目标(self, 优化目标名称: str) -> None:
        """"""
        self.优化目标 = 优化目标名称

    def 生成参数组合(self) -> list[dict]:
        """"""
        键列表: dict_keys = self.参数.keys()
        值列表: dict_values = self.参数.values()
        组合列表: list = list(product(*值列表))

        参数组合: list = []
        for 组合 in 组合列表:
            参数设置: dict = dict(zip(键列表, 组合, strict=False))
            参数组合.append(参数设置)

        return 参数组合


def 检查优化设置(
    优化设置: 类_优化设置,
    输出函数: 输出函数 = print
) -> bool:
    """"""
    if not 优化设置.生成参数组合():
        输出函数("优化参数组合为空，请检查")
        return False

    if not 优化设置.优化目标:
        输出函数("优化目标未设置，请检查")
        return False

    return True


from abc import ABC
from types import ModuleType
from typing import Optional, List, Callable
from importlib import import_module

from .模块_对象 import 类_历史数据请求, 类_行情数据, 类_K线数据
from .模块_设置 import 全局设置
from .包_国际化 import _


class 类_基础数据服务(ABC):
    """数据服务基类，用于对接不同数据源"""

    def 初始化(self, 输出函数: Callable = print) -> bool:
        """初始化数据服务连接"""
        pass

    def 查询K线历史(self, 请求: 类_历史数据请求, 输出函数: Callable = print) -> Optional[List[类_K线数据]]:
        """查询历史K线数据"""
        输出函数(_("查询K线数据失败：没有正确配置数据服务"))

    def 查询Tick历史(self, 请求: 类_历史数据请求, 输出函数: Callable = print) -> Optional[List[类_行情数据]]:
        """查询历史Tick数据"""
        输出函数(_("查询Tick数据失败：没有正确配置数据服务"))


# 全局数据服务实例
数据服务实例: 类_基础数据服务 = None


def 获取数据服务() -> 类_基础数据服务:
    """获取数据服务单例"""
    global 数据服务实例

    # 如果已经初始化则直接返回
    if 数据服务实例:
        return 数据服务实例

    # 从全局配置读取数据服务名称
    数据服务名称: str = 全局设置["数据服务.名称"]

    if not 数据服务名称:
        # 未配置时使用基础类
        数据服务实例 = 类_基础数据服务()
        print(_("没有配置要使用的数据服务，请修改全局配置中的datafeed相关内容"))
    else:
        # 构建模块名称
        模块名称: str = f"xmpy_{数据服务名称}"

        try:
            # 尝试导入数据服务模块
            模块: ModuleType = import_module(模块名称)

            # 从模块创建数据服务实例
            数据服务实例 = 模块.Datafeed()
        except ModuleNotFoundError:
            # 导入失败时回退到基础类
            数据服务实例 = 类_基础数据服务()
            print(_("无法加载数据服务模块，请运行 pip install {} 尝试安装").format(模块名称))

    return 数据服务实例


"""交易平台基础数据结构"""

from dataclasses import dataclass, field
from datetime import datetime
from logging import INFO
from typing import Optional

# from .constant import Direction 方向, Exchange 交易所, Interval 周期, Offset 开平, Status 状态, Product 产品类型, OptionType 期权类型, OrderType 委托类型
from xmpy.包_交易核心.模块_常数 import 类_方向,类_交易所,类_周期,类_开平,类_状态,类_产品类型,类_期权类型,类_委托类型
活跃状态集合 = set([类_状态.提交中, 类_状态.未成交, 类_状态.部分成交])

@dataclass
class 基础数据:
    """所有数据类的基类，包含来源网关名称"""

    网关名称: str
    扩展信息: Optional[dict] = field(default=None, init=False)


@dataclass
class 类_行情数据(基础数据):
    """行情数据，包含最新成交、盘口快照及统计信息"""

    代码: str
    交易所: 类_交易所
    时间戳: datetime

    名称: str = ""
    成交量: float = 0
    成交额: float = 0
    持仓量: float = 0
    最新价: float = 0
    最新量: float = 0
    涨停价: float = 0
    跌停价: float = 0

    开盘价: float = 0
    最高价: float = 0
    最低价: float = 0
    昨收价: float = 0

    买一价: float = 0
    买二价: float = 0
    买三价: float = 0
    买四价: float = 0
    买五价: float = 0

    卖一价: float = 0
    卖二价: float = 0
    卖三价: float = 0
    卖四价: float = 0
    卖五价: float = 0

    买一量: float = 0
    买二量: float = 0
    买三量: float = 0
    买四量: float = 0
    买五量: float = 0

    卖一量: float = 0
    卖二量: float = 0
    卖三量: float = 0
    卖四量: float = 0
    卖五量: float = 0

    本地时间: datetime = None

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"

@dataclass
class 类_K线数据(基础数据):
    """K线柱状数据"""

    代码: str
    交易所: 类_交易所
    时间戳: datetime

    周期: 类_周期 = None
    成交量: float = 0
    成交额: float = 0
    持仓量: float = 0
    开盘价: float = 0
    最高价: float = 0
    最低价: float = 0
    收盘价: float = 0

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"

@dataclass
class 类_订单数据(基础数据):
    """委托状态跟踪数据"""

    代码: str
    交易所: 类_交易所
    订单编号: str

    类型: 类_委托类型 = 类_委托类型.限价单
    方向: 类_方向 = None
    开平: 类_开平 = 类_开平.NONE
    价格: float = 0
    数量: float = 0
    已成交: float = 0
    状态: 类_状态 = 类_状态.提交中
    时间戳: datetime = None
    参考号: str = ""

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"
        self.网关_订单编号: str = f"{self.网关名称}.{self.订单编号}"

    def 是否活跃(self) -> bool:
        """检查订单是否处于活跃状态"""
        return self.状态 in 活跃状态集合

    def 创建撤单请求(self) -> "类_撤单请求":
        """生成撤单请求对象"""
        return 类_撤单请求(
            订单编号=self.订单编号,
            代码=self.代码,
            交易所=self.交易所
        )


@dataclass
class 类_成交数据(基础数据):
    """订单成交明细数据"""

    代码: str
    交易所: 类_交易所
    订单编号: str
    成交编号: str
    方向: 类_方向 = None

    开平: 类_开平 = 类_开平.NONE
    价格: float = 0
    数量: float = 0
    时间戳: datetime = None

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"
        self.网关_订单编号: str = f"{self.网关名称}.{self.订单编号}"
        self.网关_成交编号: str = f"{self.网关名称}.{self.成交编号}"


@dataclass
class 类_持仓数据(基础数据):
    """持仓信息数据"""

    代码: str
    交易所: 类_交易所
    方向: 类_方向

    数量: float = 0
    冻结量: float = 0
    价格: float = 0
    盈亏: float = 0
    昨仓量: float = 0

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"
        self.持仓_方向: str = f"{self.网关名称}.{self.代码_交易所}.{self.方向.value}"



@dataclass
class 类_账户数据(基础数据):
    """账户资金数据"""

    账户编号: str
    余额: float = 0
    冻结金额: float = 0

    def __post_init__(self) -> None:
        """计算可用资金"""
        self.可用金额: float = self.余额 - self.冻结金额
        self.账户唯一标识: str = f"{self.网关名称}.{self.账户编号}"


@dataclass
class 类_日志数据(基础数据):
    """系统日志数据"""

    消息内容: str
    日志级别: int = INFO

    def __post_init__(self) -> None:
        """自动设置记录时间"""
        self.记录时间: datetime = datetime.now()


@dataclass
class 类_合约数据(基础数据):
    """合约基本信息数据"""

    代码: str
    交易所: 类_交易所
    名称: str
    产品类型: 类_产品类型
    合约乘数: float
    最小价位: float

    最小数量: float = 1  # 最小交易手数
    最大数量: float = None  # 最大交易手数
    支持止损单: bool = False  # 是否支持止损单
    净持仓模式: bool = False  # 是否使用净持仓
    支持历史数据: bool = False  # 是否提供历史数据

    期权行权价: float = 0
    标的合约: str = ""  # 标的合约唯一标识
    期权类型: 类_期权类型 = None
    上市日期: datetime = None
    到期日期: datetime = None
    期权组合: str = ""
    期权索引: str = ""  # 相同行权价的期权标识

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"


@dataclass
class 类_报价数据(基础数据):
    """做市商报价数据"""

    代码: str
    交易所: 类_交易所
    报价编号: str

    买方价: float = 0.0
    买方量: int = 0
    卖方价: float = 0.0
    卖方量: int = 0
    买开平: 类_开平 = 类_开平.NONE
    卖开平: 类_开平 = 类_开平.NONE
    状态: 类_状态 = 类_状态.提交中
    时间戳: datetime = None
    参考号: str = ""

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"
        self.网关_报价编号: str = f"{self.网关名称}.{self.报价编号}"

    def 是否活跃(self) -> bool:
        """检查报价是否处于活跃状态"""
        return self.状态 in 活跃状态集合

    def 创建撤单请求(self) -> "类_撤单请求":
        """生成撤单请求对象"""
        return 类_撤单请求(
            订单编号=self.报价编号,
            代码=self.代码,
            交易所=self.交易所
        )


@dataclass
class 类_订阅请求:
    """行情订阅请求"""

    代码: str
    交易所: 类_交易所

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"


@dataclass
class 类_订单请求:
    """新订单请求"""

    代码: str
    交易所: 类_交易所
    方向: 类_方向
    类型: 类_委托类型
    数量: float
    价格: float = 0
    开平: 类_开平 = 类_开平.NONE
    参考号: str = ""

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"

    def 生成订单数据(self, 委托编号: str, 网关名称: str) -> 类_订单数据:
        """创建委托数据对象"""
        return 类_订单数据(
            代码=self.代码,
            交易所=self.交易所,
            订单编号=委托编号,
            类型=self.类型,
            方向=self.方向,
            开平=self.开平,
            价格=self.价格,
            数量=self.数量,
            参考号=self.参考号,
            网关名称=网关名称,
        )


@dataclass
class 类_撤单请求:
    """订单撤单请求"""

    订单编号: str
    代码: str
    交易所: 类_交易所

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"


@dataclass
class 类_历史数据请求:
    """历史数据查询请求"""

    代码: str
    交易所: 类_交易所
    开始时间: datetime
    结束时间: datetime = None
    周期: 类_周期 = None

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"


@dataclass
class 类_报价请求:
    """新报价请求"""

    代码: str
    交易所: 类_交易所
    买方价: float
    买方量: int
    卖方价: float
    卖方量: int
    买开平: 类_开平 = 类_开平.NONE
    卖开平: 类_开平 = 类_开平.NONE
    参考号: str = ""

    def __post_init__(self) -> None:
        """生成唯一标识"""
        self.代码_交易所: str = f"{self.代码}.{self.交易所.value}"

    def 生成报价数据(self, 报价编号: str, 网关名称: str) -> 类_报价数据:
        """创建报价数据对象"""
        return 类_报价数据(
            代码=self.代码,
            交易所=self.交易所,
            报价编号=报价编号,
            买方价=self.买方价,
            买方量=self.买方量,
            卖方价=self.卖方价,
            卖方量=self.卖方量,
            买开平=self.买开平,
            卖开平=self.卖开平,
            参考号=self.参考号,
            网关名称=网关名称,
        )


# 新添加
@dataclass
class 类_价差项:
    """价差数据容器"""
    名称: str
    买价: float
    卖价: float
    买量: int
    卖量: int
    净头寸: int
    时间: datetime
    价格公式: str
    交易公式: str

# @dataclass
# class 类_条件单:
#     """ 条件单 """
#     策略名称: str
#     代码_交易所: str
#     方向: Direction
#     开平: Offset
#     价格: float
#     数量: float
#     条件: 类_条件类型
#     执行价格类型: 类_执行价格类型 = 类_执行价格类型.设定价
#     分组: str = ""
#     创建时间: datetime = datetime.now()
#     触发时间: datetime = None
#     条件单编号: str = ""  # 条件单编号
#     状态: 类_条件单状态 = 类_条件单状态.等待中
#
#     def __post_init__(self):
#         """  """
#         if not self.条件单编号:
#             self.条件单编号 = datetime.now().strftime("%m%d%H%M%S%f")[:13]






"""交易平台全局配置"""

from logging import CRITICAL
from typing import Dict, Any
from tzlocal import get_localzone_name

from .模块_工具 import 加载json文件


# 全局配置字典
全局设置: Dict[str, Any] = {
    "字体.族": "微软雅黑",
    "字体.大小": 12,

    "日志.启用": True,
    "日志.级别": CRITICAL,
    "日志.控制台": True,
    "日志.文件": True,

    "邮件.服务器": "smtp.qq.com",
    "邮件.端口": 465,
    "邮件.用户名": "",
    "邮件.密码": "",
    "邮件.发件人": "",
    "邮件.收件人": "",

    "数据服务.名称": "",
    "数据服务.用户名": "",
    "数据服务.密码": "",

    "数据库.时区": get_localzone_name(),
    "数据库.类型": "sqlite",
    "数据库.文件名": "database.db",
    "数据库.主机": "",
    "数据库.端口": 0,
    "数据库.用户": "",
    "数据库.密码": ""
}


# 从JSON文件加载配置
设置文件名: str = "vt_setting.json"
全局设置.update(加载json文件(设置文件名))


def 获取配置项(前缀: str = "") -> Dict[str, Any]:
    """根据前缀筛选配置项"""
    前缀长度: int = len(前缀)
    筛选配置 = {
        键[前缀长度:]: 值
        for 键, 值 in 全局设置.items()
        if 键.startswith(前缀)
    }
    return 筛选配置


from abc import ABC, abstractmethod
from typing import Any, Dict, List
from xmpy.包_事件引擎 import 类_事件,类_事件引擎
from .模块_对象 import (
    类_行情数据, 类_成交数据, 类_订单数据, 类_持仓数据,
    类_账户数据, 类_合约数据, 类_日志数据, 类_报价数据,
    类_订阅请求, 类_订单请求, 类_撤单请求, 类_历史数据请求, 类_报价请求, 类_K线数据
)
from xmpy.包_交易核心.模块_常数 import 类_交易所
from .模块_事件类型 import (
    事件类型_行情, 事件类型_成交, 事件类型_订单,
    事件类型_持仓, 事件类型_账户, 事件类型_合约,
    事件类型_日志, 事件类型_报价
)


class 类_基础网关(ABC):
    """
    网关基类，用于对接不同交易系统的标准化接口

    实现网关时需要满足以下要求：
    1. 线程安全：所有方法需保证线程安全，对象间无可变共享状态
    2. 非阻塞：所有方法不得阻塞主线程
    3. 自动重连：网络中断后需自动尝试恢复连接
    """

    # 类属性定义
    默认名称: str = ""  # 网关默认名称
    默认配置: Dict[str, Any] = {}  # 连接所需配置模板
    支持交易所: List[类_交易所] = []  # 支持的交易所列表

    def __init__(self, 事件引擎: 类_事件引擎, 网关名称: str) -> None:
        """初始化网关实例"""
        self.事件引擎 = 事件引擎
        self.网关名称 = 网关名称

    def 触发事件(self, 事件类型: str, 事件数据: Any = None) -> None:
        """通用事件推送方法"""
        事件实例 = 类_事件(事件类型, 事件数据)
        self.事件引擎.放入事件(事件实例)

    def 推送行情(self, 行情实例: 类_行情数据) -> None:
        """处理行情数据推送"""
        # 推送全局行情事件
        self.触发事件(事件类型_行情, 行情实例)
        # 推送品种专属行情事件
        self.触发事件(f"{事件类型_行情}{行情实例.代码_交易所}", 行情实例)

    def 推送成交(self, 成交实例: 类_成交数据) -> None:
        """处理成交数据推送"""
        self.触发事件(事件类型_成交, 成交实例)
        self.触发事件(f"{事件类型_成交}{成交实例.代码_交易所}", 成交实例)

    def 推送订单(self, 订单实例: 类_订单数据) -> None:
        """处理订单状态更新"""
        self.触发事件(事件类型_订单, 订单实例)
        self.触发事件(f"{事件类型_订单}{订单实例.网关_订单编号}", 订单实例)

    def 推送持仓(self, 持仓实例: 类_持仓数据) -> None:
        """处理持仓数据更新"""
        self.触发事件(事件类型_持仓, 持仓实例)
        self.触发事件(f"{事件类型_持仓}{持仓实例.持仓_方向}", 持仓实例)

    def 推送账户(self, 账户实例: 类_账户数据) -> None:
        """处理账户资金变更"""
        self.触发事件(事件类型_账户, 账户实例)
        self.触发事件(f"{事件类型_账户}{账户实例.账户唯一标识}", 账户实例)

    def 推送报价(self, 报价实例: 类_报价数据) -> None:
        """处理报价数据更新"""
        self.触发事件(事件类型_报价, 报价实例)
        self.触发事件(f"{事件类型_报价}{报价实例.网关_报价编号}", 报价实例)

    def 记录日志(self, 日志内容: str) -> None:
        """生成日志事件"""
        日志实例 = 类_日志数据(消息内容=日志内容, 网关名称=self.网关名称)
        self.触发事件(事件类型_日志, 日志实例)

    def 推送合约(self, 合约实例: 类_合约数据) -> None:
        """处理合约信息更新"""
        self.触发事件(事件类型_合约, 合约实例)

    @abstractmethod
    def 连接(self, 连接配置: dict) -> None:
        """
        建立交易连接的核心方法

        实现要求：
        1. 完成服务器连接
        2. 连接成功后需查询并推送以下数据：
           - 合约信息：调用推送合约
           - 账户资金：调用推送账户
           - 持仓数据：调用推送持仓
           - 未完成订单：调用推送订单
           - 历史成交：调用推送成交
        3. 任一查询失败需记录错误日志
        """
        pass

    @abstractmethod
    def 断开连接(self) -> None:
        """关闭所有交易连接"""
        pass

    @abstractmethod
    def 订阅行情(self, 订阅请求: 类_订阅请求) -> None:
        """订阅指定合约的行情数据"""
        pass

    @abstractmethod
    def 发送委托(self, 订单请求: 类_订单请求) -> str:
        """
        委托下单处理

        实现要求：
        1. 根据请求创建订单数据对象
        2. 生成唯一订单编号（网关内唯一）
        3. 发送请求到交易服务器：
           - 发送成功：设置订单状态为"提交中"
           - 发送失败：设置订单状态为"已拒单"
        4. 返回格式：f"{网关名称}.{订单编号}"
        """
        pass

    @abstractmethod
    def 撤销订单(self, 撤单请求: 类_撤单请求) -> None:
        """撤销指定订单"""
        pass

    def 发送报价(self, 报价请求: 类_报价请求) -> str:
        """
        提交做市报价

        实现要求类似委托下单：
        1. 创建报价数据对象
        2. 生成唯一报价编号
        3. 处理服务器响应
        4. 返回格式：f"{网关名称}.{报价编号}"
        """
        return ""

    def 撤销报价(self, 撤单请求: 类_撤单请求) -> None:
        """撤销指定报价"""
        pass

    @abstractmethod
    def 查询账户(self) -> None:
        """主动查询账户资金信息"""
        pass

    @abstractmethod
    def 查询持仓(self) -> None:
        """主动查询持仓信息"""
        pass

    def 查询历史(self, 历史请求: 类_历史数据请求) -> List[类_K线数据]:
        """查询历史K线数据"""
        return []

    def 获取默认配置(self) -> Dict[str, Any]:
        """获取连接配置模板"""
        return self.默认配置.copy()  # 返回配置副本防止意外修改

    # 以下为需要子类维护的状态信息
    @property
    def 连接状态(self) -> bool:
        """返回当前网关连接状态（需子类实现）"""
        raise NotImplementedError

    @property
    def 交易状态(self) -> bool:
        """返回交易通道就绪状态（需子类实现）"""
        raise NotImplementedError


import time

import logging
from logging import Logger
import smtplib
import os
import traceback
from abc import ABC
from pathlib import Path
from datetime import datetime
from email.message import EmailMessage
from queue import Empty, Queue
from threading import Thread
from typing import Any, Type, Dict, List, Optional

from xmpy.包_事件引擎 import 类_事件,类_事件引擎
from .模块_应用 import 类_基础应用
from .模块_事件类型 import (
    事件类型_行情,
    事件类型_订单,
    事件类型_成交,
    事件类型_持仓,
    事件类型_账户,
    事件类型_合约,
    事件类型_日志,
    事件类型_报价
)
from .模块_网关 import 类_基础网关
from .模块_对象 import (
    类_撤单请求,
    类_日志数据,
    类_订单请求,
    类_报价数据,
    类_报价请求,
    类_订阅请求,
    类_历史数据请求,
    类_订单数据,
    类_K线数据,
    类_行情数据,
    类_成交数据,
    类_持仓数据,
    类_账户数据,
    类_合约数据,
    类_交易所
)
from .模块_设置 import 全局设置
from .模块_工具 import 获取目录路径,交易目录
from .模块_转换器 import 类_持仓转换器
from xmpy.包_交易核心.包_国际化 import _


class 类_主引擎:
    """交易平台核心引擎"""

    def __init__(self, 事件引擎: 类_事件引擎 = None) -> None:
        if 事件引擎:
            self.事件引擎: 类_事件引擎 = 事件引擎
        else:
            self.事件引擎 = 类_事件引擎()
        self.事件引擎.启动引擎()

        self.网关字典: Dict[str, 类_基础网关] = {}
        self.引擎字典: Dict[str, 基础引擎] = {}
        self.应用字典: Dict[str, 类_基础应用] = {}
        self.交易所列表: List[类_交易所] = []

        os.chdir(交易目录)
        self.初始化引擎()

    def 添加引擎(self, 引擎类: Any) -> "基础引擎":
        """添加功能引擎"""
        引擎实例: 基础引擎 = 引擎类(self, self.事件引擎)
        self.引擎字典[引擎实例.引擎名称] = 引擎实例
        return 引擎实例

    def 添加网关(self, 网关类: Type[类_基础网关], 网关名称: str = "") -> 类_基础网关:
        """添加交易通道"""
        网关名称 = 网关名称 or 网关类.默认名称
        新网关: 类_基础网关 = 网关类(self.事件引擎, 网关名称)
        self.网关字典[网关名称] = 新网关

        for 交易所 in 新网关.支持交易所:
            if 交易所 not in self.交易所列表:
                self.交易所列表.append(交易所)
        return 新网关

    def 添加应用(self, 应用类: Type[类_基础应用]) -> "基础引擎":
        """添加功能应用"""
        应用实例: 类_基础应用 = 应用类()
        self.应用字典[应用实例.应用名称] = 应用实例

        引擎实例: 基础引擎 = self.添加引擎(应用实例.引擎类)
        return 引擎实例

    def 初始化引擎(self) -> None:
        """初始化所有引擎"""
        self.添加引擎(日志引擎)
        self.添加引擎(订单管理引擎)
        self.添加引擎(邮件引擎)

    def 记录日志(self, 消息: str, 来源: str = "") -> None:
        """记录日志事件"""
        日志记录: 类_日志数据 = 类_日志数据(消息内容=消息, 网关名称=来源)
        事件实例: 类_事件 = 类_事件(事件类型_日志, 日志记录)
        self.事件引擎.放入事件(事件实例)

    def 获取网关(self, 网关名称: str) -> 类_基础网关:
        """按名称获取网关"""
        网关实例: 类_基础网关 = self.网关字典.get(网关名称, None)
        if not 网关实例:
            self.记录日志(_("找不到底层接口：{}").format(网关名称))
        return 网关实例

    def 获取引擎(self, 引擎名称: str) -> "基础引擎":
        """按名称获取引擎"""
        引擎实例: 基础引擎 = self.引擎字典.get(引擎名称, None)
        if not 引擎实例:
            self.记录日志(_("找不到引擎：{}").format(引擎名称))
        return 引擎实例

    def 获取默认配置(self, 网关名称: str) -> Optional[Dict[str, Any]]:
        """获取网关默认配置"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        return 网关实例.获取默认配置() if 网关实例 else None

    def 获取所有网关名称(self) -> List[str]:
        """获取全部网关名称"""
        return list(self.网关字典.keys())

    def 获取所有应用(self) -> List[类_基础应用]:
        """获取全部应用实例"""
        return list(self.应用字典.values())

    def 获取所有交易所(self) -> List[类_交易所]:
        """获取支持的所有交易所"""
        return self.交易所列表

    def 连接网关(self, 全局设置: dict, 网关名称: str) -> None:
        """连接指定网关"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        if 网关实例:
            网关实例.连接(全局设置)

    def 订阅行情(self, 请求: 类_订阅请求, 网关名称: str) -> None:
        """订阅行情数据"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        if 网关实例:
            网关实例.订阅行情(请求)

    def 发送委托(self, 请求: 类_订单请求, 网关名称: str) -> str:
        """委托下单"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        return 网关实例.发送委托(请求) if 网关实例 else ""

    def 撤销订单(self, 请求: 类_撤单请求, 网关名称: str) -> None:
        """撤销订单"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        if 网关实例:
            网关实例.撤销订单(请求)

    def 发送报价(self, 请求: 类_报价请求, 网关名称: str) -> str:
        """发送报价"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        return 网关实例.发送报价(请求) if 网关实例 else ""

    def 撤销报价(self, 请求: 类_撤单请求, 网关名称: str) -> None:
        """撤销报价"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        if 网关实例:
            网关实例.撤销报价(请求)

    def 查询历史(self, 请求: 类_历史数据请求, 网关名称: str) -> Optional[List[类_K线数据]]:
        """查询历史数据"""
        网关实例: 类_基础网关 = self.获取网关(网关名称)
        return 网关实例.查询历史(请求) if 网关实例 else None

    def 关闭(self) -> None:
        """关闭所有组件"""
        self.事件引擎.停止引擎()
        for 引擎 in self.引擎字典.values():
            引擎.关闭()
        for 网关 in self.网关字典.values():
            网关.断开连接()

class 基础引擎(ABC):
    """功能引擎基类"""

    def __init__(
        self,
        主引擎: 类_主引擎,
        事件引擎: 类_事件引擎,
        引擎名称: str,
    ) -> None:
        self.主引擎: 类_主引擎 = 主引擎
        self.事件引擎: 类_事件引擎 = 事件引擎
        self.引擎名称: str = 引擎名称

    def 关闭(self) -> None:
        pass

class 日志引擎(基础引擎):
    """处理日志事件"""

    def __init__(self, 主引擎: 类_主引擎, 事件引擎: 类_事件引擎) -> None:
        super().__init__(主引擎, 事件引擎, "日志")

        if not 全局设置["日志.启用"]:
            return

        self.日志级别: int = 全局设置["日志.级别"]
        self.日志记录器: Logger = logging.getLogger("veighna")
        self.日志记录器.setLevel(self.日志级别)

        self.格式器: logging.Formatter = logging.Formatter(
            "%(asctime)s  %(levelname)s: %(message)s"
        )

        self.添加空处理器()
        if 全局设置["日志.控制台"]:
            self.添加控制台处理器()
        if 全局设置["日志.文件"]:
            self.添加文件处理器()

        self.注册事件()

    def 添加空处理器(self) -> None:
        """防止无处理器错误"""
        self.日志记录器.addHandler(logging.NullHandler())

    def 添加控制台处理器(self) -> None:
        """控制台日志输出"""
        控制台处理器 = logging.StreamHandler()
        控制台处理器.setLevel(self.日志级别)
        控制台处理器.setFormatter(self.格式器)
        self.日志记录器.addHandler(控制台处理器)

    def 添加文件处理器(self) -> None:
        """文件日志输出"""
        当天日期 = datetime.now().strftime("%Y%m%d")
        日志目录:Path = 获取目录路径("log")
        文件路径:Path = 日志目录 / f"vt_{当天日期}.log"

        文件处理器 = logging.FileHandler(文件路径, mode="a", encoding="utf8")
        文件处理器.setLevel(self.日志级别)
        文件处理器.setFormatter(self.格式器)
        self.日志记录器.addHandler(文件处理器)

    def 注册事件(self) -> None:
        self.事件引擎.注册类型处理器(事件类型_日志, self.处理日志事件)

    def 处理日志事件(self, 事件: 类_事件) -> None:
        日志记录: 类_日志数据 = 事件.数据
        self.日志记录器.log(日志记录.日志级别, 日志记录.消息内容)

class 订单管理引擎(基础引擎):
    """订单管理系统"""

    def __init__(self, 主引擎: 类_主引擎, 事件引擎: 类_事件引擎) -> None:
        super().__init__(主引擎, 事件引擎, "订单管理")

        self.行情字典: Dict[str, 类_行情数据] = {}
        self.订单字典: Dict[str, 类_订单数据] = {}
        self.成交字典: Dict[str, 类_成交数据] = {}
        self.持仓字典: Dict[str, 类_持仓数据] = {}
        self.账户字典: Dict[str, 类_账户数据] = {}
        self.合约字典: Dict[str, 类_合约数据] = {}
        self.报价字典: Dict[str, 类_报价数据] = {}

        self.活跃订单字典: Dict[str, 类_订单数据] = {}
        self.活跃报价字典: Dict[str, 类_报价数据] = {}

        self.仓位转换器字典: Dict[str, 类_持仓转换器] = {}

        self.绑定查询功能()
        self.注册事件()

    def 绑定查询功能(self) -> None:
        """将查询功能绑定到主引擎"""
        self.主引擎.获取最新行情 = self.获取最新行情
        self.主引擎.获取订单详情 = self.获取订单详情
        self.主引擎.获取成交详情 = self.获取成交详情
        self.主引擎.获取持仓详情 = self.获取持仓详情
        self.主引擎.获取账户详情 = self.获取账户详情
        self.主引擎.获取合约详情 = self.获取合约详情
        self.主引擎.获取报价详情 = self.获取报价详情

        self.主引擎.获取所有行情 = self.获取所有行情
        self.主引擎.获取所有订单 = self.获取所有订单
        self.主引擎.获取所有成交 = self.获取所有成交
        self.主引擎.获取所有持仓 = self.获取所有持仓
        self.主引擎.获取所有账户 = self.获取所有账户
        self.主引擎.获取所有合约 = self.获取所有合约
        self.主引擎.获取所有报价 = self.获取所有报价
        self.主引擎.获取活跃订单 = self.获取活跃订单
        self.主引擎.获取活跃报价 = self.获取活跃报价

        self.主引擎.更新委托请求 = self.更新委托请求
        self.主引擎.转换委托请求 = self.转换委托请求
        self.主引擎.获取仓位转换器 = self.获取仓位转换器

    def 注册事件(self) -> None:
        self.事件引擎.注册类型处理器(事件类型_行情, self.处理行情事件)
        self.事件引擎.注册类型处理器(事件类型_订单, self.处理订单事件)
        self.事件引擎.注册类型处理器(事件类型_成交, self.处理成交事件)
        self.事件引擎.注册类型处理器(事件类型_持仓, self.处理持仓事件)
        self.事件引擎.注册类型处理器(事件类型_账户, self.处理账户事件)
        self.事件引擎.注册类型处理器(事件类型_合约, self.处理合约事件)
        self.事件引擎.注册类型处理器(事件类型_报价, self.处理报价事件)

    def 处理行情事件(self, 事件: 类_事件) -> None:
        行情:类_行情数据 = 事件.数据
        self.行情字典[行情.代码_交易所] = 行情

    def 处理订单事件(self, 事件: 类_事件) -> None:
        订单 = 事件.数据
        self.订单字典[订单.网关_订单编号] = 订单

        if 订单.是否活跃():
            self.活跃订单字典[订单.网关_订单编号] = 订单
        elif 订单.网关_订单编号 in self.活跃订单字典:
            self.活跃订单字典.pop(订单.网关_订单编号)

        转换器 = self.仓位转换器字典.get(订单.网关名称)
        if 转换器:
            转换器.更新委托(订单)

    def 处理成交事件(self, 事件: 类_事件) -> None:
        成交 = 事件.数据
        self.成交字典[成交.网关_成交编号] = 成交

        转换器 = self.仓位转换器字典.get(成交.网关名称)
        if 转换器:
            转换器.更新成交(成交)

    def 处理持仓事件(self, 事件: 类_事件) -> None:
        持仓 = 事件.数据
        self.持仓字典[持仓.持仓_方向] = 持仓

        转换器 = self.仓位转换器字典.get(持仓.网关名称)
        if 转换器:
            转换器.更新持仓(持仓)

    def 处理账户事件(self, 事件: 类_事件) -> None:
        账户:类_账户数据 = 事件.数据
        self.账户字典[账户.账户唯一标识] = 账户

    def 处理合约事件(self, 事件: 类_事件) -> None:
        合约: 类_合约数据 = 事件.数据
        self.合约字典[合约.代码_交易所] = 合约

        if 合约.网关名称 not in self.仓位转换器字典:
            self.仓位转换器字典[合约.网关名称] = 类_持仓转换器(self)

    def 处理报价事件(self, 事件: 类_事件) -> None:
        报价 = 事件.数据
        self.报价字典[报价.网关_报价编号] = 报价

        if 报价.是否活跃():
            self.活跃报价字典[报价.网关_报价编号] = 报价
        elif 报价.网关_报价编号 in self.活跃报价字典:
            self.活跃报价字典.pop(报价.网关_报价编号)

    # 以下为查询方法（保留原有功能，中文方法名）
    def 获取最新行情(self, 合约标识: str) -> Optional[类_行情数据]:
        return self.行情字典.get(合约标识)

    def 获取订单详情(self, 订单标识: str) -> Optional[类_订单数据]:
        return self.订单字典.get(订单标识)

    def 获取成交详情(self, 成交标识: str) -> Optional[类_成交数据]:
        return self.成交字典.get(成交标识)

    def 获取持仓详情(self, 持仓标识: str) -> Optional[类_持仓数据]:
        return self.持仓字典.get(持仓标识)

    def 获取账户详情(self, 账户标识: str) -> Optional[类_账户数据]:
        return self.账户字典.get(账户标识)

    def 获取合约详情(self, 合约标识: str) -> Optional[类_合约数据]:
        return self.合约字典.get(合约标识, None)

    def 获取报价详情(self, 报价标识: str) -> Optional[类_报价数据]:
        return self.报价字典.get(报价标识)

    def 获取所有行情(self) -> List[类_行情数据]:
        return list(self.行情字典.values())

    def 获取所有订单(self) -> List[类_订单数据]:
        return list(self.订单字典.values())

    def 获取所有成交(self) -> List[类_成交数据]:
        return list(self.成交字典.values())

    def 获取所有持仓(self) -> List[类_持仓数据]:
        return list(self.持仓字典.values())

    def 获取所有账户(self) -> List[类_账户数据]:
        return list(self.账户字典.values())

    def 获取所有合约(self) -> List[类_合约数据]:
        return list(self.合约字典.values())

    def 获取所有报价(self) -> List[类_报价数据]:
        return list(self.报价字典.values())

    def 获取活跃订单(self, 合约标识: str = "") -> List[类_订单数据]:
        if not 合约标识:
            return list(self.活跃订单字典.values())
        return [订单 for 订单 in self.活跃订单字典.values() if 订单.代码_交易所 == 合约标识]

    def 获取活跃报价(self, 合约标识: str = "") -> List[类_报价数据]:
        if not 合约标识:
            return list(self.活跃报价字典.values())
        return [报价 for 报价 in self.活跃报价字典.values() if 报价.代码_交易所 == 合约标识]

    def 更新委托请求(self, 请求: 类_订单请求, 订单标识: str, 网关名称: str) -> None:
        转换器 = self.仓位转换器字典.get(网关名称)
        if 转换器:
            转换器.更新委托请求(请求, 订单标识)

    def 转换委托请求(
        self,
        请求: 类_订单请求,
        网关名称: str,
        锁定: bool,
        净额: bool = False
    ) -> List[类_订单请求]:
        转换器 = self.仓位转换器字典.get(网关名称)
        return 转换器.转换委托请求(请求, 锁定, 净额) if 转换器 else [请求]

    def 获取仓位转换器(self, 网关名称: str) -> 类_持仓转换器:
        return self.仓位转换器字典.get(网关名称)


class 邮件引擎(基础引擎):
    """提供邮件发送功能"""

    def __init__(self, 主引擎: 类_主引擎, 事件引擎: 类_事件引擎) -> None:
        super().__init__(主引擎, 事件引擎, "邮件")

        # 邮件发送线程
        self.发送线程: Thread = Thread(target=self.运行)
        self.邮件队列: Queue = Queue()
        self.运行状态: bool = False

        # 将发送方法暴露给主引擎
        self.主引擎.发送邮件 = self.发送邮件

    def 发送邮件(self, 主题: str, 内容: str, 收件人: str = "") -> None:
        """将邮件放入发送队列"""
        # 首次发送时启动线程
        if not self.运行状态:
            self.启动()

        # 使用默认收件人
        if not 收件人:
            收件人: str = 全局设置["邮件.收件人"]

        # 构建邮件消息
        邮件消息 = EmailMessage()
        邮件消息["From"] = 全局设置["邮件.发件人"]
        邮件消息["To"] = 收件人
        邮件消息["Subject"] = 主题
        邮件消息.set_content(内容)

        self.邮件队列.put(邮件消息)

    def 运行(self) -> None:
        """邮件发送线程主循环"""
        服务器地址 = 全局设置["邮件.服务器"]
        端口号 = 全局设置["邮件.端口"]
        用户名 = 全局设置["邮件.用户名"]
        密码 = 全局设置["邮件.密码"]

        while self.运行状态:
            try:
                # 从队列获取邮件
                待发邮件 = self.邮件队列.get(block=True, timeout=1)

                try:
                    # 建立安全连接
                    with smtplib.SMTP_SSL(服务器地址, 端口号) as 邮件连接:
                        邮件连接.login(用户名, 密码)
                        邮件连接.send_message(待发邮件)
                except Exception as 异常:
                    # 记录发送失败日志
                    错误信息 = traceback.format_exc()
                    self.主引擎.记录日志(_("邮件发送失败: {}").format(错误信息), "邮件")

            except Empty:  # 空队列异常
                pass

    def 启动(self) -> None:
        """启动邮件发送线程"""
        self.运行状态 = True
        self.发送线程.start()

    def 关闭(self) -> None:
        """安全关闭邮件服务"""
        if not self.运行状态:
            return

        self.运行状态 = False
        self.发送线程.join()


"""交易平台通用常量枚举"""

from enum import Enum
from xmpy.包_交易核心.包_国际化 import _


class 类_方向(Enum):
    """委托/成交/持仓方向"""
    做多 = _("多")
    做空 = _("空")
    净头寸 = _("净")


class 类_开平(Enum):
    """委托开平仓标志"""
    NONE = ""
    开仓 = _("开")
    平仓 = _("平")
    平今 = _("平今")
    平昨 = _("平昨")


class 类_状态(Enum):
    """订单状态"""
    提交中 = _("提交中")
    未成交 = _("未成交")
    部分成交 = _("部分成交")
    全部成交 = _("全部成交")
    已撤销 = _("已撤销")
    已拒单 = _("拒单")


class 类_产品类型(Enum):
    """资产类别"""
    股票 = _("股票")
    期货 = _("期货")
    期权 = _("期权")
    指数 = _("指数")
    外汇 = _("外汇")
    现货 = _("现货")
    ETF = "ETF"
    债券 = _("债券")
    权证 = _("权证")
    价差 = _("价差")
    基金 = _("基金")
    CFD = "CFD"
    互换 = _("互换")


class 类_委托类型(Enum):
    """订单类型"""
    限价单 = _("限价")
    市价单 = _("市价")
    止损单 = "STOP"
    立即成交剩余撤销 = "FAK"
    全部成交否则撤销 = "FOK"
    询价单 = _("询价")


class 类_期权类型(Enum):
    """期权方向"""
    看涨 = _("看涨期权")
    看跌 = _("看跌期权")


class 类_交易所(Enum):
    """交易场所"""
    # 中国境内
    中金所 = "CFFEX"         # 中国金融期货交易所
    上期所 = "SHFE"           # 上海期货交易所
    郑商所 = "CZCE"           # 郑州商品交易所
    大商所 = "DCE"             # 大连商品交易所
    能源中心 = "INE"           # 上海国际能源交易中心
    广期所 = "GFEX"           # 广州期货交易所
    上交所 = "SSE"             # 上海证券交易所
    深交所 = "SZSE"           # 深圳证券交易所
    北交所 = "BSE"             # 北京证券交易所
    沪港通 = "SHHK"           # 沪深港通-上海
    深港通 = "SZHK"           # 沪深港通-深圳
    金交所 = "SGE"             # 上海黄金交易所
    无锡钢联 = "WXE"           # 无锡不锈钢交易中心
    外汇交易中心 = "CFETS"       # 中国外汇交易中心
    X债券系统 = "XBOND"         # X-Bond匿名交易系统

    # 全球市场
    智能路由 = "SMART"         # 美股智能路由
    纽交所 = "NYSE"           # 纽约证券交易所
    纳斯达克 = "NASDAQ"       # 纳斯达克交易所
    ARCA交易所 = "ARCA"        # ARCA电子交易平台
    EDGE交易所 = "EDGEA"       # Direct Edge交易所
    纳斯达克岛 = "ISLAND"      # 纳斯达克岛ECN
    BATS交易所 = "BATS"        # Bats全球市场
    IEX交易所 = "IEX"          # 投资者交易所
    美交所 = "AMEX"           # 美国证券交易所
    多伦多交易所 = "TSE"        # 多伦多证券交易所
    纽约商品 = "NYMEX"         # 纽约商业交易所
    COMEX分部 = "COMEX"       # 芝加哥商品交易所COMEX
    全球电子盘 = "GLOBEX"      # CME Globex电子平台
    外汇ECN = "IDEALPRO"     # Interactive Brokers外汇ECN
    芝加哥商品 = "CME"         # 芝加哥商品交易所
    洲际交易所 = "ICE"         # 洲际交易所集团
    港交所 = "SEHK"           # 香港联合交易所
    香港期交 = "HKFE"          # 香港期货交易所
    新交所 = "SGX"            # 新加坡交易所
    芝加哥期货 = "CBT"         # 芝加哥期货交易所
    芝加哥期权 = "CBOE"        # 芝加哥期权交易所
    CFE期货 = "CFE"           # CBOE期货交易所
    迪拜商品 = "DME"          # 迪拜商品交易所
    欧洲期货 = "EUX"          # 欧洲期货交易所
    亚太交易所 = "APEX"        # 亚太交易所
    伦敦金属 = "LME"          # 伦敦金属交易所
    大马衍生品 = "BMD"         # 马来西亚衍生品交易所
    东京商品 = "TOCOM"        # 东京工业品交易所
    泛欧交易所 = "EUNX"        # 泛欧交易所
    韩交所 = "KRX"            # 韩国交易所
    场外交易 = "OTC"          # 场外交易市场
    IB模拟盘 = "IBKRATS"      # Interactive Brokers模拟交易

    # 特殊功能
    本地数据 = "LOCAL"        # 本地生成数据


class 类_货币(Enum):
    """结算货币"""
    美元 = "USD"
    港币 = "HKD"
    人民币 = "CNY"
    加元 = "CAD"


class 类_周期(Enum):
    """K线周期"""
    一分钟 = "1m"
    一小时 = "1h"
    日线 = "d"
    周线 = "w"
    Tick级 = "tick"


from typing import Dict, List, TYPE_CHECKING
from copy import copy

from .模块_对象 import 类_合约数据, 类_订单数据, 类_成交数据, 类_持仓数据, 类_订单请求
from .模块_常数 import 类_方向, 类_开平, 类_交易所

if TYPE_CHECKING:
    from .模块_主引擎 import 类_主引擎


class 类_持仓转换器:
    """持仓转换管理器"""

    def __init__(self, 主引擎: "类_主引擎") -> None:
        self.持仓字典: Dict[str, "类_持仓明细"] = {}
        self.获取合约信息 = 主引擎.获取合约详情

    def 更新持仓(self, 持仓实例: 类_持仓数据) -> None:
        """更新持仓数据"""
        if not self.需要转换(持仓实例.代码_交易所):
            return
        持仓明细 = self.获取持仓明细(持仓实例.代码_交易所)
        持仓明细.更新持仓(持仓实例)

    def 更新成交(self, 成交实例: 类_成交数据) -> None:
        """更新成交记录"""
        if not self.需要转换(成交实例.代码_交易所):
            return
        持仓明细 = self.获取持仓明细(成交实例.代码_交易所)
        持仓明细.更新成交(成交实例)

    def 更新委托(self, 订单实例: 类_订单数据) -> None:
        """更新订单状态"""
        if not self.需要转换(订单实例.代码_交易所):
            return
        持仓明细 = self.获取持仓明细(订单实例.代码_交易所)
        持仓明细.更新委托(订单实例)

    def 更新委托请求(self, 请求: 类_订单请求, 订单标识: str) -> None:
        """处理委托请求更新"""
        if not self.需要转换(请求.代码_交易所):
            return
        持仓明细 = self.获取持仓明细(请求.代码_交易所)
        持仓明细.更新委托请求(请求, 订单标识)

    def 获取持仓明细(self, 合约标识: str) -> "类_持仓明细":
        """获取或创建持仓明细"""
        持仓明细 = self.持仓字典.get(合约标识)
        if not 持仓明细:
            合约信息 = self.获取合约信息(合约标识)
            持仓明细 = 类_持仓明细(合约信息)
            self.持仓字典[合约标识] = 持仓明细
        return 持仓明细

    def 转换委托请求(
            self,
            请求: 类_订单请求,
            锁定模式: bool,
            净仓模式: bool = False
    ) -> List[类_订单请求]:
        """转换委托请求类型"""
        if not self.需要转换(请求.代码_交易所):
            return [请求]

        持仓明细 = self.获取持仓明细(请求.代码_交易所)

        if 锁定模式:
            return 持仓明细.转换委托请求_锁定(请求)
        elif 净仓模式:
            return 持仓明细.转换委托请求_净仓(请求)
        elif 请求.交易所 in {类_交易所.上期所, 类_交易所.能源中心}:
            return 持仓明细.转换委托请求_上期所(请求)
        else:
            return [请求]

    def 需要转换(self, 合约标识: str) -> bool:
        """检查是否需要持仓转换"""
        合约信息 = self.获取合约信息(合约标识)
        return bool(合约信息 and not 合约信息.净持仓模式)


class 类_持仓明细:
    """单个合约的持仓明细管理"""

    def __init__(self, 合约信息: 类_合约数据) -> None:
        self.合约标识 = 合约信息.代码_交易所
        self.交易所 = 合约信息.交易所

        self.活跃委托字典: Dict[str, 类_订单数据] = {}

        # 多头持仓
        self.多头持仓: float = 0
        self.多头昨仓: float = 0
        self.多头今仓: float = 0

        # 空头持仓
        self.空头持仓: float = 0
        self.空头昨仓: float = 0
        self.空头今仓: float = 0

        # 冻结持仓
        self.多头冻结: float = 0
        self.多头昨冻: float = 0
        self.多头今冻: float = 0

        self.空头冻结: float = 0
        self.空头昨冻: float = 0
        self.空头今冻: float = 0

    def 更新持仓(self, 持仓实例: 类_持仓数据) -> None:
        """更新持仓基础数据"""
        if 持仓实例.方向 == 类_方向.做多:
            self.多头持仓 = 持仓实例.数量
            self.多头昨仓 = 持仓实例.昨仓量
            self.多头今仓 = self.多头持仓 - self.多头昨仓
        else:
            self.空头持仓 = 持仓实例.数量
            self.空头昨仓 = 持仓实例.昨仓量
            self.空头今仓 = self.空头持仓 - self.空头昨仓

    def 更新委托(self, 订单实例: 类_订单数据) -> None:
        """处理委托单更新"""
        if 订单实例.是否活跃():
            self.活跃委托字典[订单实例.网关_订单编号] = 订单实例
        else:
            self.活跃委托字典.pop(订单实例.网关_订单编号, None)
        self.计算冻结量()

    def 更新委托请求(self, 请求: 类_订单请求, 订单标识: str) -> None:
        """处理委托请求转换"""
        网关名称, 订单编号 = 订单标识.split(".")
        新订单 = 请求.生成订单数据(订单编号, 网关名称)
        self.更新委托(新订单)

    def 更新成交(self, 成交实例: 类_成交数据) -> None:
        """处理成交数据更新"""
        if 成交实例.方向 == 类_方向.做多:
            self.处理多头成交(成交实例)
        else:
            self.处理空头成交(成交实例)

        # 更新总持仓
        self.多头持仓 = self.多头今仓 + self.多头昨仓
        self.空头持仓 = self.空头今仓 + self.空头昨仓
        self.统计冻结总量()

    def 处理多头成交(self, 成交实例: 类_成交数据) -> None:
        """处理多头方向成交"""
        if 成交实例.开平 == 类_开平.开仓:
            self.多头今仓 += 成交实例.数量
        elif 成交实例.开平 == 类_开平.平今:
            self.空头今仓 -= 成交实例.数量
        elif 成交实例.开平 == 类_开平.平昨:
            self.空头昨仓 -= 成交实例.数量
        elif 成交实例.开平 == 类_开平.平仓:
            if 成交实例.交易所 in {类_交易所.上期所, 类_交易所.能源中心}:
                self.空头昨仓 -= 成交实例.数量
            else:
                self.空头今仓 -= 成交实例.数量
                if self.空头今仓 < 0:
                    self.空头昨仓 += self.空头今仓
                    self.空头今仓 = 0

    def 处理空头成交(self, 成交实例: 类_成交数据) -> None:
        """处理空头方向成交"""
        if 成交实例.开平 == 类_开平.开仓:
            self.空头今仓 += 成交实例.数量
        elif 成交实例.开平 == 类_开平.平今:
            self.多头今仓 -= 成交实例.数量
        elif 成交实例.开平 == 类_开平.平昨:
            self.多头昨仓 -= 成交实例.数量
        elif 成交实例.开平 == 类_开平.平仓:
            if 成交实例.交易所 in {类_交易所.上期所, 类_交易所.能源中心}:
                self.多头昨仓 -= 成交实例.数量
            else:
                self.多头今仓 -= 成交实例.数量
                if self.多头今仓 < 0:
                    self.多头昨仓 += self.多头今仓
                    self.多头今仓 = 0

    def 计算冻结量(self) -> None:
        """计算各仓位冻结数量"""
        self.重置冻结量()

        for 订单 in self.活跃委托字典.values():
            if 订单.开平 == 类_开平.开仓:
                continue

            冻结量 = 订单.数量 - 订单.已成交

            if 订单.方向 == 类_方向.做多:
                self.处理多头冻结(订单, 冻结量)
            else:
                self.处理空头冻结(订单, 冻结量)

        self.统计冻结总量()

    def 处理多头冻结(self, 订单: 类_订单数据, 冻结量: float) -> None:
        """处理多头方向冻结"""
        if 订单.开平 == 类_开平.平今:
            self.空头今冻 += 冻结量
        elif 订单.开平 == 类_开平.平昨:
            self.空头昨冻 += 冻结量
        elif 订单.开平 == 类_开平.平仓:
            self.空头今冻 += 冻结量
            if self.空头今冻 > self.空头今仓:
                self.空头昨冻 += (self.空头今冻 - self.空头今仓)
                self.空头今冻 = self.空头今仓

    def 处理空头冻结(self, 订单: 类_订单数据, 冻结量: float) -> None:
        """处理空头方向冻结"""
        if 订单.开平 == 类_开平.平今:
            self.多头今冻 += 冻结量
        elif 订单.开平 == 类_开平.平昨:
            self.多头昨冻 += 冻结量
        elif 订单.开平 == 类_开平.平仓:
            self.多头今冻 += 冻结量
            if self.多头今冻 > self.多头今仓:
                self.多头昨冻 += (self.多头今冻 - self.多头今仓)
                self.多头今冻 = self.多头今仓

    def 统计冻结总量(self) -> None:
        """统计并校验冻结总量"""
        self.多头今冻 = min(self.多头今冻, self.多头今仓)
        self.多头昨冻 = min(self.多头昨冻, self.多头昨仓)
        self.空头今冻 = min(self.空头今冻, self.空头今仓)
        self.空头昨冻 = min(self.空头昨冻, self.空头昨仓)

        self.多头冻结 = self.多头今冻 + self.多头昨冻
        self.空头冻结 = self.空头今冻 + self.空头昨冻

    def 重置冻结量(self) -> None:
        """重置所有冻结量为零"""
        self.多头冻结 = self.多头今冻 = self.多头昨冻 = 0
        self.空头冻结 = self.空头今冻 = self.空头昨冻 = 0

    def 转换委托请求_上期所(self, 请求: 类_订单请求) -> List[类_订单请求]:
        """上期所特殊平仓规则处理"""
        if 请求.开平 == 类_开平.开仓:
            return [请求]

        if 请求.方向 == 类_方向.做多:
            可用总量 = self.空头持仓 - self.空头冻结
            今仓可用 = self.空头今仓 - self.空头今冻
        else:
            可用总量 = self.多头持仓 - self.多头冻结
            今仓可用 = self.多头今仓 - self.多头今冻

        if 请求.数量 > 可用总量:
            return []
        elif 请求.数量 <= 今仓可用:
            今仓请求 = copy(请求)
            今仓请求.开平 = 类_开平.平今
            return [今仓请求]
        else:
            请求列表 = []
            if 今仓可用 > 0:
                今仓部分 = copy(请求)
                今仓部分.开平 = 类_开平.平今
                今仓部分.数量 = 今仓可用
                请求列表.append(今仓部分)

            昨仓部分 = copy(请求)
            昨仓部分.开平 = 类_开平.平昨
            昨仓部分.数量 = 请求.数量 - 今仓可用
            请求列表.append(昨仓部分)

            return 请求列表

    def 转换委托请求_锁定(self, 请求: 类_订单请求) -> List[类_订单请求]:
        """锁仓模式转换"""
        if 请求.方向 == 类_方向.做多:
            今仓数量 = self.空头今仓
            昨仓可用 = self.空头昨仓 - self.空头昨冻
        else:
            今仓数量 = self.多头今仓
            昨仓可用 = self.多头昨仓 - self.多头昨冻

        支持平昨交易所 = {类_交易所.上期所, 类_交易所.能源中心}

        if 今仓数量 and self.交易所 not in 支持平昨交易所:
            开仓请求 = copy(请求)
            开仓请求.开平 = 类_开平.开仓
            return [开仓请求]
        else:
            平仓量 = min(请求.数量, 昨仓可用)
            开仓量 = max(0, 请求.数量 - 昨仓可用)
            请求列表 = []

            if 平仓量:
                平仓请求 = copy(请求)
                if self.交易所 in 支持平昨交易所:
                    平仓请求.开平 = 类_开平.平昨
                else:
                    平仓请求.开平 = 类_开平.平仓
                平仓请求.数量 = 平仓量
                请求列表.append(平仓请求)

            if 开仓量:
                开仓请求 = copy(请求)
                开仓请求.开平 = 类_开平.开仓
                开仓请求.数量 = 开仓量
                请求列表.append(开仓请求)

            return 请求列表

    def 转换委托请求_净仓(self, 请求: 类_订单请求) -> List[类_订单请求]:
        """净仓模式转换"""
        if 请求.方向 == 类_方向.做多:
            可用总量 = self.空头持仓 - self.空头冻结
            今仓可用 = self.空头今仓 - self.空头今冻
            昨仓可用 = self.空头昨仓 - self.空头昨冻
        else:
            可用总量 = self.多头持仓 - self.多头冻结
            今仓可用 = self.多头今仓 - self.多头今冻
            昨仓可用 = self.多头昨仓 - self.多头昨冻

        if 请求.交易所 in {类_交易所.上期所, 类_交易所.能源中心}:
            return self.处理上期所净仓(请求, 可用总量, 今仓可用, 昨仓可用)
        else:
            return self.处理普通净仓(请求, 可用总量)

    def 处理上期所净仓(self, 请求: 类_订单请求, 可用总量: float, 今仓: float, 昨仓: float) -> List[类_订单请求]:
        """处理上期所净仓转换"""
        请求列表 = []
        剩余量 = 请求.数量

        if 今仓:
            今仓量 = min(今仓, 剩余量)
            剩余量 -= 今仓量
            今仓请求 = copy(请求)
            今仓请求.开平 = 类_开平.平今
            今仓请求.数量 = 今仓量
            请求列表.append(今仓请求)

        if 剩余量 and 昨仓:
            昨仓量 = min(昨仓, 剩余量)
            剩余量 -= 昨仓量
            昨仓请求 = copy(请求)
            昨仓请求.开平 = 类_开平.平昨
            昨仓请求.数量 = 昨仓量
            请求列表.append(昨仓请求)

        if 剩余量 > 0:
            开仓请求 = copy(请求)
            开仓请求.开平 = 类_开平.开仓
            开仓请求.数量 = 剩余量
            请求列表.append(开仓请求)

        return 请求列表

    def 处理普通净仓(self, 请求: 类_订单请求, 可用总量: float) -> List[类_订单请求]:
        """处理普通交易所净仓转换"""
        请求列表 = []
        剩余量 = 请求.数量

        if 可用总量:
            平仓量 = min(可用总量, 剩余量)
            剩余量 -= 平仓量
            平仓请求 = copy(请求)
            平仓请求.开平 = 类_开平.平仓
            平仓请求.数量 = 平仓量
            请求列表.append(平仓请求)

        if 剩余量 > 0:
            开仓请求 = copy(请求)
            开仓请求.开平 = 类_开平.开仓
            开仓请求.数量 = 剩余量
            请求列表.append(开仓请求)

        return 请求列表


import gettext
from pathlib import Path

# 本地化文件存放路径
本地化目录: Path = Path(__file__).parent

# 创建GNUTranslations翻译对象
翻译对象: gettext.GNUTranslations = gettext.translation("xmpy", localedir=本地化目录, fallback=True)

# 创建便捷翻译函数
_ = 翻译对象.gettext